import re, string

def is_date(token:str):
    # TODO add punctuation guard...
    token = token.translate(str.maketrans('', '', string.punctuation))
    return (len(token) == 4) and (1900 <= int(token) <= 2099)

def get_urls():
    return re.compile(
        r'(https?:\/\/(?:www\.|(?!www))[a-zA-Z0-9][a-zA-Z0-9-]+[a-zA-Z0-9]\.[^\s]{2,}|https?:\/\/(?:www\.|(?!www))'
        r'[a-zA-Z0-9]\.[^\s]{2,}|www\.[a-zA-Z0-9]\.[^\s]{2,})')

def get_emojis():
    try:
        # UCS-4
        emojis_pattern = re.compile(u'([\U00002600-\U000027BF])|([\U0001f300-\U0001f64F])|([\U0001f680-\U0001f6FF])')
    except re.error:
        # UCS-2
        emojis_pattern = re.compile(
            u'([\u2600-\u27BF])|([\uD83C][\uDF00-\uDFFF])|([\uD83D][\uDC00-\uDE4F])|([\uD83D][\uDE80-\uDEFF])')
    return emojis_pattern

def get_emoticons():
    eyes, noses, mouths = r":;8BX=", r"-~'^", r")(/\|DP"
    emoticons = "[%s][%s]?[%s]" % tuple(map(re.escape, [eyes, noses, mouths]))
    return re.compile(emoticons)

def get_hashtags():
    return re.compile(r'#\w*')

def get_reserved_words():
    return re.compile(r'\s+(RT|rt|FAV|fav)')

def get_mentions():
    return re.compile(r'@\w*')

def get_multiple_spaces():
    return re.compile(r'\s{2,}|\t')

def get_element(pattern):
    pass



import os
from .pretwita import PreTwITA

__version__ = '0.1.1'
__license__ = 'GPLv3'
__author__ = 'Andrea Failla'

PACKAGE_DIR = os.path.dirname(os.path.abspath(__file__))

__all__ = [
    'PreTwITA'
]


import string, re
from .constants import IT_STOPWORDS, IT_ABBREVIATIONS, FUNCTIONS
from .patterns import get_multiple_spaces, get_urls, get_emoticons, get_emojis, \
                    get_hashtags, get_mentions, get_reserved_words, is_date  

class PreTwITA:
    def __init__(self, text: str):       
        self.text = text

    def clean(self, placeholder=False, additional_stopwords=None, keep_dates=False):
        """
        Full tweet cleaning pipeline

        :param placeholder: if true, replace patterns with placeholders instead of removing, defaults to False
        :type placeholder: bool, optional
        :param additional_stopwords: a list of additional stopwords to remove, defaults to None
        :type additional_stopwords: list[str], optional
        :param keep_dates: choice to remove/keep dates while removing numbers, defaults to False
        :type keep_dates: bool, optional
        :return: A clean PreTwITA object
        """        
        return self \
            .to_lower() \
            .correct_abbreviations() \
            .remove_urls(placeholder=placeholder) \
            .remove_emojis(placeholder=placeholder) \
            .remove_emoticons(placeholder=placeholder) \
            .remove_mentions(placeholder=placeholder) \
            .remove_hashtags(placeholder=placeholder) \
            .remove_reserved_words(placeholder=placeholder) \
            .remove_stopwords(additional_stopwords=additional_stopwords) \
            .remove_punctuation() \
            .remove_numbers(keep_dates=keep_dates) \
            .remove_multiple_spaces()

    def to_lower(self):
        self.text = self.text.lower()
        return self

    def correct_abbreviations(self):
        tokens = self.text.split()
        out_tokens_list = []
        for token in tokens:
            stripped = token.translate(str.maketrans('', '', string.punctuation))
            if stripped in IT_ABBREVIATIONS.keys():
                out_tokens_list.append(IT_ABBREVIATIONS[stripped])
            else:
                out_tokens_list.append(token)
        self.text = ' '.join(out_tokens_list)
        return self

    def remove_urls(self, placeholder=False):
        if placeholder:
            self.text = re.sub(pattern=get_urls(), repl='xxURLxx', string=self.text)
        else:
            self.text = re.sub(pattern=get_urls(), repl='', string=self.text)
        return self

    def remove_emojis(self, placeholder=False):
        if placeholder:
            self.text = re.sub(pattern=get_emojis(), repl='xxEMOJIxx ', string=self.text)
        else:
            self.text = re.sub(pattern=get_emojis(), repl=' ', string=self.text)

        return self

    def remove_emoticons(self, placeholder=False):
        if placeholder:
            self.text = re.sub(pattern=get_emoticons(), repl='xxEMOTICONxx', string=self.text)
        else:
            self.text = re.sub(pattern=get_emoticons(), repl='', string=self.text)
        return self

    def remove_mentions(self, placeholder=False):
        if placeholder:
            self.text = re.sub(pattern=get_mentions(), repl='xxMENTIONxx', string=self.text)
        else:
            self.text = re.sub(pattern=get_mentions(), repl='', string=self.text)
        return self

    def remove_hashtags(self, placeholder=False):
        if placeholder:
            self.text = re.sub(pattern=get_hashtags(), repl='xxHASHTAGxx', string=self.text)
        else:
            self.text = re.sub(pattern=get_hashtags(), repl='', string=self.text)
        return self

    def remove_reserved_words(self, placeholder=False):
        if placeholder:
            self.text = re.sub(pattern=get_reserved_words(), repl=' xxRESERVEDxx', string=self.text)
        else:
            self.text = re.sub(pattern=get_reserved_words(), repl=' ', string=self.text)
        return self

    def remove_stopwords(self, additional_stopwords=None):
        if additional_stopwords is None:
            additional_stopwords = []
        tokens = self.text.split()
        stop_words = IT_STOPWORDS + additional_stopwords
        out_tokens_list = []
        for token in tokens:
            if token not in stop_words:
                out_tokens_list.append(token)
        self.text = ' '.join(out_tokens_list)
        return self
   
    def remove_punctuation(self):
        self.text = self.text.translate(str.maketrans('', '', string.punctuation))
        return self 

    def remove_numbers(self, keep_dates=False):
        tokens = self.text.split()
        for token in tokens:
            if token.isnumeric():
                if keep_dates:
                    if not is_date(token):
                        tokens.remove(token)
                else:
                    tokens.remove(token)
        self.text = ' '.join(tokens)
        return self
    
    def remove_multiple_spaces(self):
        self.text = re.sub(pattern=get_multiple_spaces(), repl=' ', string=self.text)
        return self

    def get_tokens(self) -> list:
        return self.text.split()

    def __str__(self):
        return self.text

    def available_functions():
        for funct in FUNCTIONS:
            print(funct) 
    


IT_STOPWORDS = [
            "ad","al","allo","ai",
            "agli","all","agl","alla",
            "alle","con","col","coi",
            "da","dal","dallo","dai",
            "dagli","dall","dagl","dalla",
            "dalle","di","del","dello",
            "dei","degli","dell","degl",
            "della","delle","in","nel",
            "nello","nei","negli","nell",
            "negl","nella","nelle","su",
            "sul","sullo","sui","sugli",
            "sull","sugl","sulla","sulle",
            "per","tra","contro","io",
            "tu","lui","lei","noi",
            "voi","loro","mio","mia",
            "miei","mie","tuo","tua",
            "tuoi","tue","suo","sua",
            "suoi","sue","nostro","nostra",
            "nostri","nostre","vostro","vostra",
            "vostri","vostre","mi","ti",
            "ci","vi","lo","la",
            "li","le","gli","ne",
            "il","un","uno","una",
            "ma","ed","se","perché",
            "anche","come","dov","dove",
            "che","chi","cui","non",
            "più","quale","quanto","quanti","quanta",
            "quante","quello","quelli","quella",
            "quelle","questo","questi","questa",
            "queste","si","tutto","tutti",
            "a","c","e","i",
            "l","o","ho","hai",
            "ha","abbiamo","avete","hanno",
            "abbia","abbiate","abbiano","avrò","avrai",
            "avrà","avremo","avrete","avranno","avrei",
            "avresti","avrebbe","avremmo","avreste",
            "avrebbero","avevo","avevi","aveva",
            "avevamo","avevate","avevano","ebbi",
            "avesti","ebbe","avemmo","aveste",
            "ebbero","avessi","avesse","avessimo",
            "avessero","avendo","avuto","avuta",
            "avuti","avute","sono","sei",
            "è","siamo","siete","sia","siate",
            "siano","sarò","sarai","sarà",
            "saremo","sarete","saranno","sarei",
            "saresti","sarebbe","saremmo","sareste",
            "sarebbero","ero","eri","era",
            "eravamo","eravate","erano","fui",
            "fosti","fu","fummo","foste",
            "furono","fossi","fosse","fossimo",
            "fossero","essendo","faccio","fai",
            "facciamo","fanno","faccia","facciate",
            "facciano","farò","farai","farà",
            "faremo","farete","faranno","farei",
            "faresti","farebbe","faremmo","fareste",
            "farebbero","facevo","facevi","faceva",
            "facevamo","facevate","facevano","feci",
            "facesti","fece","facemmo","faceste",
            "fecero","facessi","facesse","facessimo",
            "facessero","facendo","sto","stai",
            "sta","stiamo","stanno","stia",
            "stiate","stiano","starò","starai","starà",
            "staremo","starete","staranno","starei",
            "staresti","starebbe","staremmo","stareste",
            "starebbero","stavo","stavi","stava",
            "stavamo","stavate","stavano","stetti",
            "stesti","stette","stemmo","steste",
            "stettero","stessi","stesse","stessimo",
            "stessero","stando", "ogni"
        ]

IT_ABBREVIATIONS = {
                    'xò':'però', 'nn':'non', 'xk':'perché',
                    'xke':'perché', 'xkè':'perché',
                    'cmq':'comunque', 'pfv':'per favore', 
                    'grz':'grazie', 'tnt':'tanto', 
                    'risp':'rispondi', 'prox':'prossimo', 
                    'gg':'giorno', 'sn':'sono', 
                    'tvb':'ti voglio bene', 'vvb':'vi voglio bene',
                    'bn':'bene', 'vbn':'va bene',
                    'prg':'prego'
                }
FUNCTIONS = [
            "clean(placeholder=False, additional_stopwords=None, keep_dates=False)",
            "to_lower()",
            "correct_abbreviations()",
            "remove_urls(placeholder=False)",
            "remove_emojis(placeholder=False)",
            "remove_emoticons(placeholder=False)",
            "remove_mentions(placeholder=False)",
            "remove_hashtags(placeholder=False)",
            "remove_reserved_words(placeholder=False)",
            "remove_stopwords(additional_stopwords=None)",
            "remove_punctuation()",
            "remove_numbers(keep_dates=False)",
            "remove_multiple_spaces()",
            "get_tokens()"
        ]


from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="pretwita",
    version="0.1.1",
    author="Andrea Failla",
    author_email="andrea.failla.ak@gmail.com",
    description="A text PREprocessor for TWeets in the ITAlian language",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/andreafailla/pretwita",
    packages=find_packages(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    keywords='nlp natural-language-processing twitter italian tweet-preprocessing',
    python_requires='>=3.0',
)